@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-encodings-dialog.c 6379 2008-08-10 16:40:21Z pborelli $
+ * $Id: gedit-encodings-dialog.c 6631 2008-11-29 11:06:49Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +34,7 @@
 
 #include <string.h>
 
-#include <glib/gslist.h>
+#include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
