@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-plugins-engine.c 6498 2008-09-07 13:50:57Z pborelli $
+ * $Id: gedit-plugins-engine.c 6639 2008-11-30 15:12:38Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,18 +42,22 @@
 #include "gedit-plugin.h"
 #include "gedit-debug.h"
 #include "gedit-app.h"
+#include "gedit-plugin-loader.h"
+#include "gedit-object-module.h"
 
-#include "gedit-module.h"
-#ifdef ENABLE_PYTHON
-#include "gedit-python-module.h"
-#endif
-
-#define USER_GEDIT_PLUGINS_LOCATION ".gnome2/gedit/plugins/"
+#define USER_GEDIT_LOCATION ".gnome2/gedit/"
 
 #define GEDIT_PLUGINS_ENGINE_BASE_KEY "/apps/gedit-2/plugins"
 #define GEDIT_PLUGINS_ENGINE_KEY GEDIT_PLUGINS_ENGINE_BASE_KEY "/active-plugins"
 
 #define PLUGIN_EXT	".gedit-plugin"
+#define LOADER_EXT	G_MODULE_SUFFIX
+
+typedef struct
+{
+	GeditPluginLoader *loader;
+	GeditObjectModule *module;
+} LoaderInfo;
 
 /* Signals */
 enum
@@ -70,7 +74,10 @@ G_DEFINE_TYPE(GeditPluginsEngine, gedit_plugins_engine, G_TYPE_OBJECT)
 struct _GeditPluginsEnginePrivate
 {
 	GList *plugin_list;
+	GHashTable *loaders;
+
 	GConfClient *gconf_client;
+	gboolean activate_from_gconf;
 };
 
 GeditPluginsEngine *default_engine = NULL;
@@ -84,17 +91,21 @@ static void	gedit_plugins_engine_activate_plugin_real (GeditPluginsEngine *engin
 static void	gedit_plugins_engine_deactivate_plugin_real (GeditPluginsEngine *engine,
 							     GeditPluginInfo    *info);
 
-static void
-gedit_plugins_engine_load_dir (GeditPluginsEngine *engine,
-			       const gchar        *dir,
-			       GSList             *active_plugins)
+typedef gboolean (*LoadDirCallback)(GeditPluginsEngine *engine, const gchar *filename, gpointer userdata);
+
+static gboolean
+load_dir_real (GeditPluginsEngine *engine,
+	       const gchar        *dir,
+	       const gchar        *suffix,
+	       LoadDirCallback     callback,
+	       gpointer            userdata)
 {
 	GError *error = NULL;
 	GDir *d;
 	const gchar *dirent;
+	gboolean ret = TRUE;
 
-	g_return_if_fail (engine->priv->gconf_client != NULL);
-	g_return_if_fail (dir != NULL);
+	g_return_val_if_fail (dir != NULL, TRUE);
 
 	gedit_debug_message (DEBUG_PLUGINS, "DIR: %s", dir);
 
@@ -103,67 +114,78 @@ gedit_plugins_engine_load_dir (GeditPluginsEngine *engine,
 	{
 		g_warning ("%s", error->message);
 		g_error_free (error);
-		return;
+		return TRUE;
 	}
-
+	
 	while ((dirent = g_dir_read_name (d)))
 	{
-		if (g_str_has_suffix (dirent, PLUGIN_EXT))
-		{
-			gchar *plugin_file;
-			GeditPluginInfo *info;
-			
-			plugin_file = g_build_filename (dir, dirent, NULL);
-			info = _gedit_plugin_info_new (plugin_file);
-			g_free (plugin_file);
+		gchar *filename;
 
-			if (info == NULL)
-				continue;
+		if (!g_str_has_suffix (dirent, suffix))
+			continue;
 
-			/* If a plugin with this name has already been loaded
-			 * drop this one (user plugins override system plugins) */
-			if (gedit_plugins_engine_get_plugin_info (engine, info->module_name) != NULL)
-			{
-				g_warning ("Two or more plugins named '%s'. "
-					   "Only the first will be considered.\n",
-					   info->module_name);
+		filename = g_build_filename (dir, dirent, NULL);
 
-				_gedit_plugin_info_unref (info);
+		ret = callback (engine, filename, userdata);
 
-				continue;
-			}
+		g_free (filename);
 
-			/* Actually, the plugin will be activated when reactivate_all
-			 * will be called for the first time. */
-			info->active = g_slist_find_custom (active_plugins,
-							    info->module_name,
-							    (GCompareFunc)strcmp) != NULL;
-
-			engine->priv->plugin_list = g_list_prepend (engine->priv->plugin_list, info);
-
-			gedit_debug_message (DEBUG_PLUGINS, "Plugin %s loaded", info->name);
-		}
+		if (!ret)
+			break;
 	}
 
 	g_dir_close (d);
+	return ret;
+}
+
+static gboolean
+load_plugin_info (GeditPluginsEngine *engine,
+		  const gchar        *filename,
+		  gpointer            userdata)
+{
+	GeditPluginInfo *info;
+	
+	info = _gedit_plugin_info_new (filename);
+
+	if (info == NULL)
+		return TRUE;
+
+	/* If a plugin with this name has already been loaded
+	 * drop this one (user plugins override system plugins) */
+	if (gedit_plugins_engine_get_plugin_info (engine, gedit_plugin_info_get_module_name (info)) != NULL)
+	{
+		g_warning ("Two or more plugins named '%s'. "
+			   "Only the first will be considered.\n",
+			   gedit_plugin_info_get_module_name (info));
+
+		_gedit_plugin_info_unref (info);
+
+		return TRUE;
+	}
+
+	engine->priv->plugin_list = g_list_prepend (engine->priv->plugin_list, info);
+
+	gedit_debug_message (DEBUG_PLUGINS, "Plugin %s loaded", info->name);
+	return TRUE;
 }
 
 static void
-gedit_plugins_engine_load_all (GeditPluginsEngine *engine)
+load_all_real (GeditPluginsEngine *engine,
+               const gchar        *dir,
+               const gchar        *envname,
+               const gchar        *envdefault,
+               const gchar        *suffix,
+	       LoadDirCallback     callback,
+	       gpointer            userdata)
 {
-	GSList *active_plugins = NULL;
 	const gchar *home;
-	const gchar *pdirs_env;
+	const gchar *pdirs_env = NULL;
 	gchar **pdirs;
 	int i;
 
-	active_plugins = gconf_client_get_list (engine->priv->gconf_client,
-						GEDIT_PLUGINS_ENGINE_KEY,
-						GCONF_VALUE_STRING,
-						NULL);
-
 	/* load user's plugins */
 	home = g_get_home_dir ();
+
 	if (home == NULL)
 	{
 		g_warning ("Could not get HOME directory\n");
@@ -171,31 +193,95 @@ gedit_plugins_engine_load_all (GeditPluginsEngine *engine)
 	else
 	{
 		gchar *pdir;
+		gboolean ret = TRUE;
 
 		pdir = g_build_filename (home,
-					 USER_GEDIT_PLUGINS_LOCATION,
+					 USER_GEDIT_LOCATION,
+					 dir,
 					 NULL);
 
 		if (g_file_test (pdir, G_FILE_TEST_IS_DIR))
-			gedit_plugins_engine_load_dir (engine, pdir, active_plugins);
+			ret = load_dir_real (engine, pdir, suffix, callback, userdata);
 		
 		g_free (pdir);
+		
+		if (!ret)
+			return;
 	}
+	
+	if (envname)
+		pdirs_env = g_getenv (envname);
 
-	pdirs_env = g_getenv ("GEDIT_PLUGINS_PATH");
-	/* What if no env var is set? We use the default location(s)! */
 	if (pdirs_env == NULL)
-		pdirs_env = GEDIT_PLUGINDIR;
+		pdirs_env = envdefault;
 
-	gedit_debug_message (DEBUG_PLUGINS, "GEDIT_PLUGINS_PATH=%s", pdirs_env);
+	gedit_debug_message (DEBUG_PLUGINS, "%s=%s", envname, pdirs_env);
 	pdirs = g_strsplit (pdirs_env, G_SEARCHPATH_SEPARATOR_S, 0);
 
 	for (i = 0; pdirs[i] != NULL; i++)
-		gedit_plugins_engine_load_dir (engine, pdirs[i], active_plugins);
+	{
+		if (!load_dir_real (engine, pdirs[i], suffix, callback, userdata))
+			break;
+	}
 
 	g_strfreev (pdirs);
-	g_slist_foreach (active_plugins, (GFunc) g_free, NULL);
-	g_slist_free (active_plugins);
+}
+
+static void
+load_all_plugins (GeditPluginsEngine *engine)
+{
+	load_all_real (engine, 
+		       "plugins", 
+		       "GEDIT_PLUGINS_PATH", 
+		       GEDIT_PLUGINDIR,
+		       PLUGIN_EXT,
+		       load_plugin_info,
+		       NULL);
+}
+
+static guint
+hash_lowercase (gconstpointer data)
+{
+	gchar *lowercase;
+	guint ret;
+	
+	lowercase = g_ascii_strdown ((const gchar *)data, -1);
+	ret = g_str_hash (lowercase);
+	g_free (lowercase);
+	
+	return ret;
+}
+
+static gboolean
+equal_lowercase (gconstpointer a, gconstpointer b)
+{
+	return g_ascii_strcasecmp ((const gchar *)a, (const gchar *)b) == 0;
+}
+
+static void
+loader_destroy (LoaderInfo *info)
+{
+	if (!info)
+		return;
+	
+	if (info->loader)
+		g_object_unref (info->loader);
+	
+	g_free (info);
+}
+
+static void
+add_loader (GeditPluginsEngine *engine,
+	    const gchar        *loader_id,
+	    GeditObjectModule  *module)
+{
+	LoaderInfo *info;
+
+	info = g_new (LoaderInfo, 1);
+	info->loader = NULL;
+	info->module = module;
+
+	g_hash_table_insert (engine->priv->loaders, g_strdup (loader_id), info);
 }
 
 static void
@@ -226,46 +312,57 @@ gedit_plugins_engine_init (GeditPluginsEngine *engine)
 				 gedit_plugins_engine_active_plugins_changed,
 				 engine, NULL, NULL);
 
-	gedit_plugins_engine_load_all (engine);
+	load_all_plugins (engine);
+	
+	/* make sure that the first reactivation will read active plugins from
+	   gconf */
+	engine->priv->activate_from_gconf = TRUE;
+
+	/* mapping from loadername -> loader object */
+	engine->priv->loaders = g_hash_table_new_full (hash_lowercase,
+						       equal_lowercase,
+						       (GDestroyNotify)g_free,
+						       (GDestroyNotify)loader_destroy);
+}
+
+static void
+loader_garbage_collect (const char *id, LoaderInfo *info)
+{
+	if (info->loader)
+		gedit_plugin_loader_garbage_collect (info->loader);
 }
 
 void
 gedit_plugins_engine_garbage_collect (GeditPluginsEngine *engine)
 {
-	GType *module_types = g_type_children (GEDIT_TYPE_MODULE, NULL);
-	unsigned i;
-	for (i = 0; module_types[i] != 0; i++)
-	{
-		gpointer klass = g_type_class_peek (module_types[i]);
-		if (klass != NULL)
-			gedit_module_class_garbage_collect (klass);
-	}
-	g_free (module_types);
+	g_hash_table_foreach (engine->priv->loaders,
+			      (GHFunc) loader_garbage_collect,
+			      NULL);
 }
 
 static void
 gedit_plugins_engine_finalize (GObject *object)
 {
 	GeditPluginsEngine *engine = GEDIT_PLUGINS_ENGINE (object);
-
+	GList *item;
+	
 	gedit_debug (DEBUG_PLUGINS);
 
-#ifdef ENABLE_PYTHON
-	/* Note: that this may cause finalization of objects (typically
-	 * the GeditWindow) by running the garbage collector. Since some
-	 * of the plugin may have installed callbacks upon object
-	 * finalization (typically they need to free the WindowData)
-	 * it must run before we get rid of the plugins.
-	 */
-	gedit_python_shutdown ();
-#endif
-
 	g_return_if_fail (engine->priv->gconf_client != NULL);
 
-	g_list_foreach (engine->priv->plugin_list,
-			(GFunc) _gedit_plugin_info_unref, NULL);
+	for (item = engine->priv->plugin_list; item; item = item->next)
+	{
+		GeditPluginInfo *info = GEDIT_PLUGIN_INFO (item->data);
+		
+		if (gedit_plugin_info_is_active (info))
+			gedit_plugins_engine_deactivate_plugin_real (engine, info);
+		
+		_gedit_plugin_info_unref (info);
+	}
+	
 	g_list_free (engine->priv->plugin_list);
-
+	
+	g_hash_table_destroy (engine->priv->loaders);
 	g_object_unref (engine->priv->gconf_client);
 }
 
@@ -304,6 +401,109 @@ gedit_plugins_engine_class_init (GeditPluginsEngineClass *klass)
 	g_type_class_add_private (klass, sizeof (GeditPluginsEnginePrivate));
 }
 
+static gboolean
+load_loader (GeditPluginsEngine *engine,
+	     const gchar        *filename,
+	     gpointer		 data)
+{
+	GeditObjectModule *module;
+	gchar *base;
+	gchar *path;
+	const gchar *id;
+	GType type;
+	
+	/* try to load in the module */
+	path = g_path_get_dirname (filename);
+	base = g_path_get_basename (filename);
+
+	module = gedit_object_module_new (base, path, "register_gedit_plugin_loader");
+	
+	g_free (base);
+	g_free (path);
+
+	/* make sure to load the type definition */
+	if (!g_type_module_use (G_TYPE_MODULE (module)))
+	{
+		g_object_unref (module);
+		g_warning ("Plugin loader module `%s' could not be loaded", filename);
+
+		return TRUE;
+	}
+
+	/* get the exported type and check the name as exported by the 
+	 * loader interface */
+	type = gedit_object_module_get_object_type (module);
+	id = gedit_plugin_loader_type_get_id (type);
+	
+	add_loader (engine, id, module);	
+	g_type_module_unuse (G_TYPE_MODULE (module));
+
+	return TRUE;
+}
+
+static void
+ensure_loader (LoaderInfo *info)
+{
+	if (info->loader == NULL && info->module != NULL)
+	{
+		/* create a new loader object */
+		GeditPluginLoader *loader;
+		loader = (GeditPluginLoader *)gedit_object_module_new_object (info->module, NULL);
+	
+		if (loader == NULL || !GEDIT_IS_PLUGIN_LOADER (loader))
+		{
+			g_warning ("Loader object is not a valid GeditPluginLoader instance");
+		
+			if (loader != NULL && G_IS_OBJECT (loader))
+				g_object_unref (loader);
+		}
+		else
+		{
+			info->loader = loader;
+		}
+	}
+}
+
+static GeditPluginLoader *
+get_plugin_loader (GeditPluginsEngine *engine, GeditPluginInfo *info)
+{
+	const gchar *loader_id;
+	LoaderInfo *loader_info;
+
+	loader_id = info->loader;
+
+	loader_info = (LoaderInfo *)g_hash_table_lookup (
+			engine->priv->loaders, 
+			loader_id);
+
+	if (loader_info == NULL)
+	{
+		/* loader could not be found in the hash, try to find it by 
+		   scanning */
+		load_all_real (engine, 
+			       "plugin-loaders", 
+			       NULL, 
+			       GEDIT_LOADERDIR,
+			       LOADER_EXT,
+			       (LoadDirCallback)load_loader,
+			       NULL);
+		
+		loader_info = (LoaderInfo *)g_hash_table_lookup (
+				engine->priv->loaders, 
+				loader_id);
+	}
+
+	if (loader_info == NULL)
+	{
+		/* cache non-existent so we don't scan again */
+		add_loader (engine, loader_id, NULL);
+		return NULL;
+	}
+	
+	ensure_loader (loader_info);
+	return loader_info->loader;
+}
+
 GeditPluginsEngine *
 gedit_plugins_engine_get_default (void)
 {
@@ -328,7 +528,7 @@ static gint
 compare_plugin_info_and_name (GeditPluginInfo *info,
 			      const gchar *module_name)
 {
-	return strcmp (info->module_name, module_name);
+	return strcmp (gedit_plugin_info_get_module_name (info), module_name);
 }
 
 GeditPluginInfo *
@@ -341,67 +541,6 @@ gedit_plugins_engine_get_plugin_info (GeditPluginsEngine *engine,
 	return l == NULL ? NULL : (GeditPluginInfo *) l->data;
 }
 
-static gboolean
-load_plugin_module (GeditPluginInfo *info)
-{
-	gchar *dirname;
-
-	gedit_debug (DEBUG_PLUGINS);
-
-	g_return_val_if_fail (info != NULL, FALSE);
-	g_return_val_if_fail (info->file != NULL, FALSE);
-	g_return_val_if_fail (info->module_name != NULL, FALSE);
-	g_return_val_if_fail (info->plugin == NULL, FALSE);
-	g_return_val_if_fail (info->available, FALSE);
-
-	dirname = g_path_get_dirname (info->file);
-	g_return_val_if_fail (dirname != NULL, FALSE);
-
-#ifdef ENABLE_PYTHON
-	if (info->module_type == GEDIT_TYPE_PYTHON_MODULE)
-	{
-		if (!gedit_python_init ())
-		{
-			/* Mark plugin as unavailable and fail */
-			info->available = FALSE;
-			g_warning ("Cannot load Python plugin '%s' since gedit "
-			           "was not able to initialize the Python interpreter.",
-			           info->name);
-		}
-	}
-#endif
-
-	info->module = GEDIT_MODULE (g_object_new (info->module_type,
-						   "path", dirname,
-						   "module-name", info->module_name,
-						   NULL));
-
-	g_free (dirname);
-
-	if (!g_type_module_use (G_TYPE_MODULE (info->module)))
-	{
-		g_warning ("Cannot load plugin '%s' since file '%s' cannot be read.",
-			   gedit_module_get_module_name (info->module),
-			   gedit_module_get_path (info->module));
-
-		g_object_unref (G_OBJECT (info->module));
-		info->module = NULL;
-
-		/* Mark plugin as unavailable and fail. */
-		info->available = FALSE;
-		
-		return FALSE;
-	}
-
-	info->plugin = GEDIT_PLUGIN (gedit_module_new_object (info->module));
-
-	g_type_module_unuse (G_TYPE_MODULE (info->module));
-
-	gedit_debug_message (DEBUG_PLUGINS, "End");
-
-	return TRUE;
-}
-
 static void
 save_active_plugin_list (GeditPluginsEngine *engine)
 {
@@ -411,11 +550,12 @@ save_active_plugin_list (GeditPluginsEngine *engine)
 
 	for (l = engine->priv->plugin_list; l != NULL; l = l->next)
 	{
-		const GeditPluginInfo *info = (const GeditPluginInfo *) l->data;
-		if (info->active)
+		GeditPluginInfo *info = (GeditPluginInfo *) l->data;
+
+		if (gedit_plugin_info_is_active (info))
 		{
 			active_plugins = g_slist_prepend (active_plugins,
-							  info->module_name);
+							  (gpointer)gedit_plugin_info_get_module_name (info));
 		}
 	}
 
@@ -431,28 +571,56 @@ save_active_plugin_list (GeditPluginsEngine *engine)
 	g_slist_free (active_plugins);
 }
 
+static gboolean
+load_plugin (GeditPluginsEngine *engine,
+	     GeditPluginInfo    *info)
+{
+	GeditPluginLoader *loader;
+	gchar *path;
+
+	if (gedit_plugin_info_is_active (info))
+		return TRUE;
+	
+	if (!gedit_plugin_info_is_available (info))
+		return FALSE;
+
+	loader = get_plugin_loader (engine, info);
+	
+	if (loader == NULL)
+	{
+		g_warning ("Could not find loader `%s' for plugin `%s'", info->loader, info->name);
+		info->available = FALSE;
+		return FALSE;
+	}
+	
+	path = g_path_get_dirname (info->file);
+	g_return_val_if_fail (path != NULL, FALSE);
+
+	info->plugin = gedit_plugin_loader_load (loader, info, path);
+	
+	g_free (path);
+	
+	if (info->plugin == NULL)
+	{
+		g_warning ("Error loading plugin '%s'", info->name);
+		info->available = FALSE;
+		return FALSE;
+	}
+	
+	return TRUE;
+}
+
 static void
 gedit_plugins_engine_activate_plugin_real (GeditPluginsEngine *engine,
 					   GeditPluginInfo *info)
 {
-	gboolean res = TRUE;
-
-	if (info->active || !info->available)
+	if (!load_plugin (engine, info))
 		return;
 
-	if (info->plugin == NULL)
-		res = load_plugin_module (info);
-
-	if (res)
-	{
-		const GList *wins = gedit_app_get_windows (gedit_app_get_default ());
-		for (; wins != NULL; wins = wins->next)
-			gedit_plugin_activate (info->plugin, GEDIT_WINDOW (wins->data));
-
-		info->active = TRUE;
-	}
-	else
-		g_warning ("Error activating plugin '%s'", info->name);
+	/* activate plugin for all windows */
+	const GList *wins = gedit_app_get_windows (gedit_app_get_default ());
+	for (; wins != NULL; wins = wins->next)
+		gedit_plugin_activate (info->plugin, GEDIT_WINDOW (wins->data));
 }
 
 gboolean
@@ -463,17 +631,18 @@ gedit_plugins_engine_activate_plugin (GeditPluginsEngine *engine,
 
 	g_return_val_if_fail (info != NULL, FALSE);
 
-	if (!info->available)
+	if (!gedit_plugin_info_is_available (info))
 		return FALSE;
 		
-	if (info->active)
+	if (gedit_plugin_info_is_active (info))
 		return TRUE;
 
 	g_signal_emit (engine, signals[ACTIVATE_PLUGIN], 0, info);
-	if (info->active)
+
+	if (gedit_plugin_info_is_active (info))
 		save_active_plugin_list (engine);
 
-	return info->active;
+	return gedit_plugin_info_is_active (info);
 }
 
 static void
@@ -481,15 +650,26 @@ gedit_plugins_engine_deactivate_plugin_real (GeditPluginsEngine *engine,
 					     GeditPluginInfo *info)
 {
 	const GList *wins;
+	GeditPluginLoader *loader;
 
-	if (!info->active || !info->available)
+	if (!gedit_plugin_info_is_active (info) || 
+	    !gedit_plugin_info_is_available (info))
 		return;
 
 	wins = gedit_app_get_windows (gedit_app_get_default ());
 	for (; wins != NULL; wins = wins->next)
 		gedit_plugin_deactivate (info->plugin, GEDIT_WINDOW (wins->data));
 
-	info->active = FALSE;
+	/* first unref the plugin (the loader still has one) */
+	g_object_unref (info->plugin);
+	
+	/* find the loader and tell it to gc and unload the plugin */
+	loader = get_plugin_loader (engine, info);
+	
+	gedit_plugin_loader_garbage_collect (loader);
+	gedit_plugin_loader_unload (loader, info);
+	
+	info->plugin = NULL;
 }
 
 gboolean
@@ -500,40 +680,91 @@ gedit_plugins_engine_deactivate_plugin (GeditPluginsEngine *engine,
 
 	g_return_val_if_fail (info != NULL, FALSE);
 
-	if (!info->active || !info->available)
+	if (!gedit_plugin_info_is_active (info))
 		return TRUE;
 
 	g_signal_emit (engine, signals[DEACTIVATE_PLUGIN], 0, info);
-	if (!info->active)
+	if (!gedit_plugin_info_is_active (info))
 		save_active_plugin_list (engine);
 
-	return !info->active;
+	return !gedit_plugin_info_is_active (info);
 }
 
-static void
-reactivate_all (GeditPluginsEngine *engine,
-		GeditWindow        *window)
+void
+gedit_plugins_engine_activate_plugins (GeditPluginsEngine *engine,
+					GeditWindow        *window)
 {
 	GList *pl;
+	GSList *active_plugins = NULL;
 
 	gedit_debug (DEBUG_PLUGINS);
 
+	g_return_if_fail (GEDIT_IS_PLUGINS_ENGINE (engine));
+	g_return_if_fail (GEDIT_IS_WINDOW (window));
+	
+	/* the first time, we get the 'active' plugins from gconf */
+	if (engine->priv->activate_from_gconf)
+	{
+		active_plugins = gconf_client_get_list (engine->priv->gconf_client,
+							GEDIT_PLUGINS_ENGINE_KEY,
+							GCONF_VALUE_STRING,
+							NULL);
+	}
+
 	for (pl = engine->priv->plugin_list; pl; pl = pl->next)
 	{
-		gboolean res = TRUE;
-		
 		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
+		
+		if (engine->priv->activate_from_gconf && 
+		    g_slist_find_custom (active_plugins,
+					 gedit_plugin_info_get_module_name (info),
+					 (GCompareFunc)strcmp) == NULL)
+			continue;
+		
+		/* If plugin is not active, don't try to activate/load it */
+		if (!engine->priv->activate_from_gconf && 
+		    !gedit_plugin_info_is_active (info))
+			continue;
 
-		/* If plugin is not available, don't try to activate/load it */
-		if (info->available && info->active)
-		{		
-			if (info->plugin == NULL)
-				res = load_plugin_module (info);
-				
-			if (res)
-				gedit_plugin_activate (info->plugin,
-						       window);			
-		}
+		if (load_plugin (engine, info))
+			gedit_plugin_activate (info->plugin,
+					       window);
+	}
+	
+	if (engine->priv->activate_from_gconf)
+	{
+		g_slist_foreach (active_plugins, (GFunc) g_free, NULL);
+		g_slist_free (active_plugins);
+		engine->priv->activate_from_gconf = FALSE;
+	}
+	
+	gedit_debug_message (DEBUG_PLUGINS, "End");
+
+	/* also call update_ui after activation */
+	gedit_plugins_engine_update_plugins_ui (engine, window);
+}
+
+void
+gedit_plugins_engine_deactivate_plugins (GeditPluginsEngine *engine,
+					  GeditWindow        *window)
+{
+	GList *pl;
+	
+	gedit_debug (DEBUG_PLUGINS);
+
+	g_return_if_fail (GEDIT_IS_PLUGINS_ENGINE (engine));
+	g_return_if_fail (GEDIT_IS_WINDOW (window));
+	
+	for (pl = engine->priv->plugin_list; pl; pl = pl->next)
+	{
+		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
+		
+		/* check if the plugin is actually active */
+		if (!gedit_plugin_info_is_active (info))
+			continue;
+		
+		/* call deactivate for the plugin for this window */
+		gedit_plugin_deactivate (info->plugin, window);
 	}
 	
 	gedit_debug_message (DEBUG_PLUGINS, "End");
@@ -541,28 +772,24 @@ reactivate_all (GeditPluginsEngine *engine,
 
 void
 gedit_plugins_engine_update_plugins_ui (GeditPluginsEngine *engine,
-					GeditWindow        *window,
-					gboolean            new_window)
+					 GeditWindow        *window)
 {
 	GList *pl;
 
 	gedit_debug (DEBUG_PLUGINS);
 
+	g_return_if_fail (GEDIT_IS_PLUGINS_ENGINE (engine));
 	g_return_if_fail (GEDIT_IS_WINDOW (window));
 
-	if (new_window)
-		reactivate_all (engine, window);
-
-	/* updated ui of all the plugins that implement update_ui */
+	/* call update_ui for all active plugins */
 	for (pl = engine->priv->plugin_list; pl; pl = pl->next)
 	{
 		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
 
-		if (!info->available || !info->active)
+		if (!gedit_plugin_info_is_active (info))
 			continue;
 			
 	       	gedit_debug_message (DEBUG_PLUGINS, "Updating UI of %s", info->name);
-		
 		gedit_plugin_update_ui (info->plugin, window);
 	}
 }
@@ -633,16 +860,16 @@ gedit_plugins_engine_active_plugins_changed (GConfClient *client,
 	{
 		GeditPluginInfo *info = (GeditPluginInfo*)pl->data;
 
-		if (!info->available)
+		if (!gedit_plugin_info_is_available (info))
 			continue;
 
 		to_activate = (g_slist_find_custom (active_plugins,
-						    info->module_name,
+						    gedit_plugin_info_get_module_name (info),
 						    (GCompareFunc)strcmp) != NULL);
 
-		if (!info->active && to_activate)
+		if (!gedit_plugin_info_is_active (info) && to_activate)
 			g_signal_emit (engine, signals[ACTIVATE_PLUGIN], 0, info);
-		else if (info->active && !to_activate)
+		else if (gedit_plugin_info_is_active (info) && !to_activate)
 			g_signal_emit (engine, signals[DEACTIVATE_PLUGIN], 0, info);
 	}
 
