@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c 6133 2008-01-27 15:02:26Z pborelli $ 
+ * $Id: gedit-view.c 6606 2008-11-22 15:02:10Z icq $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -95,6 +95,8 @@ struct _GeditViewPrivate
 	guint        search_entry_changed_id;
 	
 	gboolean     disable_popdown;
+	
+	GtkTextBuffer *current_buffer;
 };
 
 /* The search entry completion is shared among all the views */
@@ -136,7 +138,11 @@ static void 	search_highlight_updated_cb	(GeditDocument    *doc,
 						 GtkTextIter      *start,
 						 GtkTextIter      *end,
 						 GeditView        *view);
-						 
+
+static void	gedit_view_delete_from_cursor 	(GtkTextView     *text_view,
+						 GtkDeleteType    type,
+						 gint             count);
+
 G_DEFINE_TYPE(GeditView, gedit_view, GTK_TYPE_SOURCE_VIEW)
 
 /* Signals */
@@ -174,6 +180,8 @@ gedit_view_class_init (GeditViewClass *klass)
 	GObjectClass     *object_class = G_OBJECT_CLASS (klass);
 	GtkObjectClass   *gtkobject_class = GTK_OBJECT_CLASS (klass);
 	GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (klass);
+	GtkTextViewClass *text_view_class = GTK_TEXT_VIEW_CLASS (klass);
+	
 	GtkBindingSet    *binding_set;
 
 	gtkobject_class->destroy = gedit_view_destroy;
@@ -204,6 +212,8 @@ gedit_view_class_init (GeditViewClass *klass)
 	klass->start_interactive_goto_line = start_interactive_goto_line;
 	klass->reset_searched_text = reset_searched_text;	
 
+	text_view_class->delete_from_cursor = gedit_view_delete_from_cursor;
+	
 	view_signals[START_INTERACTIVE_SEARCH] =
     		g_signal_new ("start_interactive_search",
 		  	      G_TYPE_FROM_CLASS (object_class),
@@ -248,20 +258,32 @@ gedit_view_class_init (GeditViewClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__BOXED,
 			      G_TYPE_NONE, 1, G_TYPE_STRV);
-
+			      
 	g_type_class_add_private (klass, sizeof (GeditViewPrivate));
 	
 	binding_set = gtk_binding_set_by_class (klass);
 	
-	gtk_binding_entry_add_signal (binding_set, GDK_k, GDK_CONTROL_MASK, "start_interactive_search", 0);
-	gtk_binding_entry_add_signal (binding_set, GDK_K, GDK_CONTROL_MASK, "start_interactive_search", 0);
+	gtk_binding_entry_add_signal (binding_set,
+				      GDK_k,
+				      GDK_CONTROL_MASK,
+				      "start_interactive_search", 0);
+		
+	gtk_binding_entry_add_signal (binding_set,
+				      GDK_i,
+				      GDK_CONTROL_MASK,
+				      "start_interactive_goto_line", 0);
 	
-	gtk_binding_entry_add_signal (binding_set, GDK_i, GDK_CONTROL_MASK, "start_interactive_goto_line", 0);
-	gtk_binding_entry_add_signal (binding_set, GDK_I, GDK_CONTROL_MASK, "start_interactive_goto_line", 0);
-	
-	gtk_binding_entry_add_signal (binding_set, GDK_k, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "reset_searched_text", 0);
-	gtk_binding_entry_add_signal (binding_set, GDK_K, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "reset_searched_text", 0);	
+	gtk_binding_entry_add_signal (binding_set,
+				      GDK_k,
+				      GDK_CONTROL_MASK | GDK_SHIFT_MASK,
+				      "reset_searched_text", 0);
 
+	gtk_binding_entry_add_signal (binding_set, 
+				      GDK_d, 
+				      GDK_CONTROL_MASK,
+				      "delete_from_cursor", 2,
+				      G_TYPE_ENUM, GTK_DELETE_PARAGRAPHS,
+				      G_TYPE_INT, 1);
 }
 
 static gboolean
@@ -273,6 +295,51 @@ scroll_to_cursor_on_init (GeditView *view)
 	return FALSE;
 }
 
+static void
+current_buffer_removed (GeditView *view)
+{
+	if (view->priv->current_buffer)
+	{
+		g_signal_handlers_disconnect_by_func (view->priv->current_buffer,
+						      document_read_only_notify_handler,
+						      view);
+		g_signal_handlers_disconnect_by_func (view->priv->current_buffer,
+						      search_highlight_updated_cb,
+						      view);
+				     
+		g_object_unref (view->priv->current_buffer);
+		view->priv->current_buffer = NULL;
+	}
+}
+
+static void
+on_notify_buffer_cb (GeditView  *view,
+		     GParamSpec *arg1,
+		     gpointer    userdata)
+{
+	GtkTextBuffer *buffer;
+	
+	current_buffer_removed (view);
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	
+	if (buffer == NULL || !GEDIT_IS_DOCUMENT (buffer))
+		return;
+
+	view->priv->current_buffer = g_object_ref (buffer);
+	g_signal_connect (buffer,
+			  "notify::read-only",
+			  G_CALLBACK (document_read_only_notify_handler),
+			  view);
+
+	gtk_text_view_set_editable (GTK_TEXT_VIEW (view), 
+				    !gedit_document_get_readonly (GEDIT_DOCUMENT (buffer)));
+
+	g_signal_connect (buffer,
+			  "search_highlight_updated",
+			  G_CALLBACK (search_highlight_updated_cb),
+			  view);
+}
+
 static void 
 gedit_view_init (GeditView *view)
 {
@@ -331,6 +398,12 @@ gedit_view_init (GeditView *view)
 
 	if (tl != NULL)
 		gtk_target_list_add_uri_targets (tl, TARGET_URI_LIST);
+		
+	/* Act on buffer change */
+	g_signal_connect (view, 
+			  "notify::buffer", 
+			  G_CALLBACK (on_notify_buffer_cb),
+			  NULL);
 }
 
 static void
@@ -359,6 +432,12 @@ gedit_view_destroy (GtkObject *object)
 		}
 	}
 	
+	/* Disconnect notify buffer because the destroy of the textview will
+	   set the buffer to NULL, and we call get_buffer in the notify which
+	   would reinstate a GtkTextBuffer which we don't want */
+	current_buffer_removed (view);
+	g_signal_handlers_disconnect_by_func (view, on_notify_buffer_cb, NULL);
+	
 	(* GTK_OBJECT_CLASS (gedit_view_parent_class)->destroy) (object);
 }
 
@@ -369,6 +448,8 @@ gedit_view_finalize (GObject *object)
 
 	view = GEDIT_VIEW (object);
 
+	current_buffer_removed (view);
+
 	g_free (view->priv->old_search_text);
 
 	(* G_OBJECT_CLASS (gedit_view_parent_class)->finalize) (object);
@@ -408,23 +489,7 @@ gedit_view_new (GeditDocument *doc)
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
 
-	view = GTK_WIDGET (g_object_new (GEDIT_TYPE_VIEW, NULL));
-	
-	gtk_text_view_set_buffer (GTK_TEXT_VIEW (view),
-				  GTK_TEXT_BUFFER (doc));
-  		
-	g_signal_connect (doc,
-			  "notify::read-only",
-			  G_CALLBACK (document_read_only_notify_handler),
-			  view);
-
-	gtk_text_view_set_editable (GTK_TEXT_VIEW (view), 
-				    !gedit_document_get_readonly (doc));					  
-
-	g_signal_connect (doc,
-			  "search_highlight_updated",
-			  G_CALLBACK (search_highlight_updated_cb),
-			  view);
+	view = GTK_WIDGET (g_object_new (GEDIT_TYPE_VIEW, "buffer", doc, NULL));
 
 	gedit_debug_message (DEBUG_VIEW, "END: %d", G_OBJECT (view)->ref_count);
 
@@ -1188,12 +1253,24 @@ search_entry_insert_text (GtkEditable *editable,
 		gunichar c;
 		const gchar *p;
 	 	const gchar *end;
+	 	const gchar *next;
 
 		p = text;
 		end = text + length;
 
-		while (p != end) {
-			const gchar *next;
+		if (p == end)
+			return;
+
+		c = g_utf8_get_char (p);
+		
+		if ((c == '-' || c == '+') && *position == 0)
+		{
+			next = g_utf8_next_char (p);
+			p = next;
+		}
+
+		while (p != end)
+		{
 			next = g_utf8_next_char (p);
 
 			c = g_utf8_get_char (p);
@@ -1586,8 +1663,31 @@ search_init (GtkWidget *entry,
 		{
 			gboolean moved;
 			gint line;
-
-			line = MAX (atoi (entry_text) - 1, 0);
+			gint offset_line = 0;
+			
+			if (*entry_text == '-')
+			{
+				gint cur_line = gtk_text_iter_get_line (&view->priv->start_search_iter);
+			
+				if (*(entry_text + 1) != '\0')
+					offset_line = MAX (atoi (entry_text + 1), 0);
+				
+				line = MAX (cur_line - offset_line, 0);
+			}
+			else if (*entry_text == '+')
+			{
+				gint cur_line = gtk_text_iter_get_line (&view->priv->start_search_iter);
+			
+				if (*(entry_text + 1) != '\0')
+					offset_line = MAX (atoi (entry_text + 1), 0);
+				
+				line = cur_line + offset_line;
+			}
+			else
+			{
+				line = MAX (atoi (entry_text) - 1, 0);
+			}
+			
 			moved = gedit_document_goto_line (doc, line);
 			gedit_view_scroll_to_cursor (view);
 
@@ -1862,3 +1962,120 @@ search_highlight_updated_cb (GeditDocument *doc,
 	}
 }
 
+/* There is no "official" way to reset the im context in GtkTextView */
+static void
+reset_im_context (GtkTextView *text_view)
+{
+	if (text_view->need_im_reset)
+	{
+		text_view->need_im_reset = FALSE;
+		gtk_im_context_reset (text_view->im_context);
+	}
+}
+
+static void
+delete_line (GtkTextView *text_view,
+	     gint         count)
+{
+	GtkTextIter start;
+	GtkTextIter end;
+	GtkTextBuffer *buffer;
+
+	buffer = gtk_text_view_get_buffer (text_view);
+
+	reset_im_context (text_view);
+
+	/* If there is a selection delete the selected lines and
+	 * ignore count */
+	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
+	{
+		gtk_text_iter_order (&start, &end);
+		
+		if (gtk_text_iter_starts_line (&end))
+		{
+			/* Do no delete the line with the cursor if the cursor
+			 * is at the beginning of the line */
+			count = 0;
+		}
+		else	
+			count = 1;
+	}
+	
+	gtk_text_iter_set_line_offset (&start, 0);
+
+	if (count > 0)
+	{		
+		gtk_text_iter_forward_lines (&end, count);
+
+		if (gtk_text_iter_is_end (&end))
+		{		
+			if (gtk_text_iter_backward_line (&start) && !gtk_text_iter_ends_line (&start))
+				gtk_text_iter_forward_to_line_end (&start);
+		}
+	}
+	else if (count < 0) 
+	{
+		if (!gtk_text_iter_ends_line (&end))
+			gtk_text_iter_forward_to_line_end (&end);
+
+		while (count < 0) 
+		{
+			if (!gtk_text_iter_backward_line (&start))
+				break;
+				
+			++count;
+		}
+
+		if (count == 0)
+		{
+			if (!gtk_text_iter_ends_line (&start))
+				gtk_text_iter_forward_to_line_end (&start);
+		}
+		else
+			gtk_text_iter_forward_line (&end);
+	}
+
+	if (!gtk_text_iter_equal (&start, &end))
+	{
+		GtkTextIter cur = start;
+		gtk_text_iter_set_line_offset (&cur, 0);
+		
+		gtk_text_buffer_begin_user_action (buffer);
+
+		gtk_text_buffer_place_cursor (buffer, &cur);
+
+		gtk_text_buffer_delete_interactive (buffer, 
+						    &start,
+						    &end,
+						    gtk_text_view_get_editable (text_view));
+
+		gtk_text_buffer_end_user_action (buffer);
+
+		gtk_text_view_scroll_mark_onscreen (text_view,
+						    gtk_text_buffer_get_insert (buffer));
+	}
+	else
+	{
+		gtk_widget_error_bell (GTK_WIDGET (text_view));
+	}
+}
+
+static void
+gedit_view_delete_from_cursor (GtkTextView   *text_view,
+			       GtkDeleteType  type,
+			       gint           count)
+{
+	/* We override the standard handler for delete_from_cursor since
+	   the GTK_DELETE_PARAGRAPHS case is not implemented as we like (i.e. it
+	   does not remove the carriage return in the previous line)
+	 */
+	switch (type)
+	{
+		case GTK_DELETE_PARAGRAPHS:
+			delete_line (text_view, count);
+			break;
+		default:
+			GTK_TEXT_VIEW_CLASS (gedit_view_parent_class)->delete_from_cursor(text_view, type, count);
+			break;
+	}
+}
